@@ -314,62 +314,26 @@ class ObjManager(object):
                                                         self.show_progress):
             continue
 
-    def _compute_total_size(self, obj):
-        pending_descendents = list(obj.children)
-        seen = _intset.IDSet()
-        seen.add(obj.address)
-        total_size = obj.size
-        while pending_descendents:
-            next_ref = pending_descendents.pop()
-            if next_ref in seen:
-                continue
-            seen.add(next_ref)
-            next_obj = self.objs.get(next_ref, None)
-            if next_obj is None:
-                continue
-            # type and frame types tend to cause us to recurse into
-            # everything. So for now, when we encounter them, don't add
-            # their references
-            total_size += next_obj.size
-            pending_descendents.extend([ref for ref in next_obj.children
-                                             if ref not in seen])
-        ## count = len(seen)
-        ## # This single object references more than 10% of all objects, and
-        ## # expands to more that 10x its direct references
-        ## if count > obj.num_refs * 10 and count > break_on:
-        ##     import pdb; pdb.set_trace()
-        obj.total_size = total_size
+    def compute_total_size(self, obj):
+        """Sum the size of all referenced objects (recursively)."""
+        obj.total_size = sum(c.size for c in obj.iter_recursive_refs())
         return obj
 
-    def compute_total_size(self):
-        """This computes the total bytes referenced from this object."""
-        # Unfortunately, this is an N^2 operation :(. The problem is that
-        # a.total_size + b.total_size != c.total_size (if c references a & b).
-        # This is because a & b may refer to common objects. Consider something
-        # like:
-        #   A   _
-        #  / \ / \
-        # B   C  |
-        #  \ /  /
-        #   D--'
-        # D & C participate in a refcycle, and B has an alternative path to D.
-        # You certainly don't want to count D 2 times when computing the total
-        # size of A. Also, how do you give the relative contribution of B vs C
-        # in this graph?
-        total = len(self.objs)
-        break_on = total / 10
-        for idx, obj in enumerate(self.objs.itervalues()):
-            if self.show_progress and idx & 0x1ff == 0:
-                sys.stderr.write('compute size %8d / %8d        \r'
-                                 % (idx, total))
-            self._compute_total_size(obj)
-        if self.show_progress:
-            sys.stderr.write('compute size %8d / %8d        \n'
-                             % (idx, total))
+    def summarize(self, obj=None, excluding=None):
+        """Summarize the objects referenced from this one.
 
-    def summarize(self):
+        :param obj: Given obj as the root object, aggregate the count and size
+            of the types of each referenced object.
+            If not supplied, we will walk all objects.
+        :param excluding: A list of addresses to exclude from the aggregate
+        :return: An _ObjSummary() of this subset of the graph
+        """
         summary = _ObjSummary()
-        for obj in self.objs.itervalues():
+        if obj is None:
+            objs = self.objs.itervalues()
+        else:
+            objs = obj.iter_recursive_refs(excluding=excluding)
+        for obj in objs:
             summary._add(obj)
         return summary
 
@@ -544,8 +508,13 @@ def load(source, using_json=None, show_prog=True, collapse=True):
         if cleanup is not None:
             cleanup()
     if collapse:
+        tstart = time.time()
         manager.compute_parents()
         manager.collapse_instance_dicts()
+        if show_prog:
+            tend = time.time()
+            sys.stderr.write('collapsed in %.1fs\n'
+                             % (tend - tstart,))
     return manager
 
 
