@@ -18,6 +18,7 @@ import sys
 
 from meliae import (
     _loader,
+    _scanner,
     warn,
     tests,
     )
@@ -192,6 +193,70 @@ class TestMemObjectCollection(tests.TestCase):
         self.assertEqual([1024, 512], list(moc))
         self.assertEqual([1024, 512], list(moc.iterkeys()))
 
+    def assertSizeOf(self, num_words, obj, extra_size=0, has_gc=True):
+        expected_size = extra_size + num_words * _scanner._word_size
+        if has_gc:
+            expected_size += _scanner._gc_head_size
+        self.assertEqual(expected_size, _scanner.size_of(obj))
+
+    def test__sizeof__base(self):
+        moc = _loader.MemObjectCollection()
+        # Empty table size is 1024 pointers
+        # 1: PyType*
+        # 2: refcnt
+        # 3: vtable*
+        # 4: _table*
+        # 3 4-byte int attributes
+        self.assertSizeOf(4+1024, moc, extra_size=3*4, has_gc=False)
+
+    def test__sizeof__one_item(self):
+        moc = _loader.MemObjectCollection()
+        # We also track the size of the referenced _MemObject entries
+        # Which is:
+        # 1: PyObject *address
+        # 2: PyObject *type_str
+        # 3: long size
+        # 4: *child_list
+        # 5: *value
+        # 6: *parent_list
+        # 7: ulong total_size
+        # 8: *proxy
+        moc.add(0, 'foo', 100)
+        self.assertSizeOf(4+1024+8, moc, extra_size=3*4, has_gc=False)
+
+    def test__sizeof__with_reflists(self):
+        moc = _loader.MemObjectCollection()
+        # We should assign the memory for ref-lists to the container. A
+        # ref-list allocates the number of entries + 1
+        # Each _memobject also takes up
+        moc.add(0, 'foo', 100, children=[1234], parent_list=[3456, 7890])
+        self.assertSizeOf(4+1024+8+2+3, moc, extra_size=3*4, has_gc=False)
+
+    def test__sizeof__with_dummy(self):
+        moc = _loader.MemObjectCollection()
+        moc.add(0, 'foo', 100, children=[1234], parent_list=[3456, 7890])
+        moc.add(1, 'foo', 100, children=[1234], parent_list=[3456, 7890])
+        del moc[1]
+        self.assertSizeOf(4+1024+8+2+3, moc, extra_size=3*4, has_gc=False)
+
+    def test_traverse_empty(self):
+        # With nothing present, we return no referents
+        moc = _loader.MemObjectCollection()
+        self.assertEqual([], _scanner.get_referents(moc))
+
+    def test_traverse_simple_item(self):
+        moc = _loader.MemObjectCollection()
+        moc.add(1234, 'foo', 100)
+        self.assertEqual([1234, 'foo', None], _scanner.get_referents(moc))
+
+    def test_traverse_multiple_and_parents_and_children(self):
+        moc = _loader.MemObjectCollection()
+        moc.add(1, 'foo', 100)
+        moc.add(2, 'bar', 200, children=[8, 9], parent_list=[10, 11],
+                value='test val')
+        self.assertEqual([1, 'foo', None, 2, 'bar', 'test val', 8, 9, 10, 11],
+                         _scanner.get_referents(moc))
+
 
 class Test_MemObjectProxy(tests.TestCase):
 
@@ -457,3 +522,131 @@ class Test_MemObjectProxy(tests.TestCase):
         mop = self.moc.add(3, 'dict', 140, children=[2, 1])
         as_dict = mop.refs_as_dict()
         self.assertEqual({'a': mop_f}, as_dict)
+
+    def assertSizeOf(self, num_words, obj, extra_size=0, has_gc=True):
+        expected_size = extra_size + num_words * _scanner._word_size
+        if has_gc:
+            expected_size += _scanner._gc_head_size
+        self.assertEqual(expected_size, _scanner.size_of(obj))
+
+    def test__sizeof__(self):
+        mop = self.moc[0]
+        # 1: PyType*
+        # 2: refcnt
+        # 3: collection*
+        # 4: _MemObject*
+        # 5: _managed_obj *
+        # No vtable because we have no cdef functions
+        self.assertSizeOf(5, mop, has_gc=True)
+
+    def test__sizeof__managed(self):
+        mop = self.moc[0]
+        del self.moc[0]
+        # If the underlying object has been removed from the collection, then
+        # the memory is now being managed by the mop itself. So it grows by:
+        # 1: PyObject* address
+        # 2: PyObject* type_str
+        # 3: long size
+        # 4: RefList *child_list
+        # 5: PyObject *value
+        # 6: RefList *parent_list
+        # 7: unsigned long total_size
+        # 8: PyObject *proxy
+        self.assertSizeOf(5+8, mop, has_gc=True)
+
+    def test_traverse(self):
+        # When a Proxied object is removed from its Collection, it becomes
+        # owned by the Proxy itself, and should be returned from tp_traverse
+        mop = self.moc[0]
+        # At this point, moc still controls the _MemObject
+        self.assertEqual([self.moc], _scanner.get_referents(mop))
+        # But now, it references everything else, too
+        del self.moc[0]
+        self.assertEqual([self.moc, mop.address, mop.type_str, mop.value],
+                         _scanner.get_referents(mop))
+
+    def test_traverse_with_parent_and_children(self):
+        mop = self.moc.add(1, 'my_class', 1234, children=[5,6,7],
+                           parent_list=[8,9,10], value='test str')
+        # While in the moc, the Collection manages the references
+        self.assertEqual([self.moc], _scanner.get_referents(mop))
+        # But once gone, we refer to everything directly
+        del self.moc[1]
+        self.assertEqual([self.moc, mop.address, mop.type_str, mop.value,
+                          5, 6, 7, 8, 9, 10],
+                         _scanner.get_referents(mop))
+
+    def test_compute_total_size(self):
+        obj = self.moc.add(1, 'test', 1234, children=[0])
+        obj = self.moc.add(2, 'other', 4567, children=[1])
+        self.assertEqual(1234+4567+100, obj.compute_total_size())
+        self.assertEqual(1234+4567, obj.compute_total_size(excluding=[0]))
+
+    def test_all(self):
+        self.moc.add(1, 'foo', 1234, children=[0])
+        obj = self.moc.add(2, 'other', 4567, children=[1])
+        self.assertEqual([1, 0], [o.address for o in obj.all('foo')])
+        self.assertEqual([1],
+                         [o.address for o in obj.all('foo', excluding=[0])])
+        # Excluding 1 actually prevents us from walking further
+        self.assertEqual([],
+                         [o.address for o in obj.all('foo', excluding=[1])])
+
+
+class Test_MemObjectProxyIterRecursiveRefs(tests.TestCase):
+
+    def setUp(self):
+        super(Test_MemObjectProxyIterRecursiveRefs, self).setUp()
+        self.moc = _loader.MemObjectCollection()
+        self.moc.add(1024, 'bar', 200)
+        self.moc.add(0, 'foo', 100)
+        self.moc.add(255, 'baz', 300)
+
+    def assertIterRecursiveRefs(self, addresses, obj, excluding=None):
+        self.assertEqual(addresses,
+            [o.address for o in obj.iter_recursive_refs(excluding=excluding)])
+
+    def test_no_refs(self):
+        self.assertIterRecursiveRefs([1024], self.moc[1024])
+
+    def test_single_depth_refs(self):
+        obj = self.moc.add(1, 'test', 1234, children=[1024])
+        self.assertIterRecursiveRefs([1, 1024], obj)
+
+    def test_deep_refs(self):
+        obj = self.moc.add(1, '1', 1234, children=[2])
+        self.moc.add(2, '2', 1234, children=[3])
+        self.moc.add(3, '3', 1234, children=[4])
+        self.moc.add(4, '4', 1234, children=[5])
+        self.moc.add(5, '5', 1234, children=[6])
+        self.moc.add(6, '6', 1234, children=[])
+        self.assertIterRecursiveRefs([1, 2, 3, 4, 5, 6], obj)
+
+    def test_self_referenced(self):
+        self.moc.add(1, 'test', 1234, children=[1024, 2])
+        obj = self.moc.add(2, 'test2', 1234, children=[1])
+        self.assertIterRecursiveRefs([2, 1, 1024], obj)
+
+    def test_excluding(self):
+        obj = self.moc.add(1, 'test', 1234, children=[1024])
+        self.assertIterRecursiveRefs([1], obj, excluding=[1024])
+
+    def test_excluding_nested(self):
+        self.moc.add(1, 'test', 1234, children=[1024])
+        obj = self.moc.add(2, 'test', 1234, children=[1])
+        self.assertIterRecursiveRefs([2, 1, 1024], obj)
+        self.assertIterRecursiveRefs([2], obj, excluding=[1])
+
+    def test_excluding_wraparound(self):
+        self.moc.add(1, 'test', 1234, children=[1024])
+        self.moc.add(2, 'test', 1234, children=[1024])
+        obj = self.moc.add(3, 'test', 1234, children=[1, 2])
+        self.assertIterRecursiveRefs([3, 2, 1024, 1], obj)
+        self.assertIterRecursiveRefs([3, 2, 1024], obj, excluding=[1])
+        refed = (o.address for o in self.moc[1].iter_recursive_refs())
+        self.assertIterRecursiveRefs([3, 2], obj, excluding=refed)
+
+    def test_excluding_self(self):
+        self.assertIterRecursiveRefs([], self.moc[1024], excluding=[1024])
+        obj = self.moc.add(1, '1', 1234, children=[1024])
+        self.assertIterRecursiveRefs([], obj, excluding=[1])
