@@ -93,7 +93,7 @@ _intern_dict_dump = [
 '{"address": 4, "type": "str", "size": 25, "len": 1, "value": "c", "refs": []}',
 '{"address": 5, "type": "str", "size": 25, "len": 1, "value": "d", "refs": []}',
 '{"address": 6, "type": "dict", "size": 512, "refs": [2, 5, 5, 5, 4, 4, 3, 3]}',
-'{"address": 7, "type": "dict", "size": 512, "refs": [8, 8, 5, 5, 4, 4, 3, 3]}',
+'{"address": 7, "type": "dict", "size": 512, "refs": [6, 6, 5, 5, 4, 4, 3, 3]}',
 '{"address": 8, "type": "dict", "size": 512, "refs": [2, 2, 5, 5, 4, 4, 3, 3]}',
 ]
 
@@ -281,25 +281,20 @@ class TestObjManager(tests.TestCase):
 
     def test_compute_total_size(self):
         manager = loader.load(_example_dump, show_prog=False)
-        manager.compute_total_size()
         objs = manager.objs
+        manager.compute_total_size(objs[1])
         self.assertEqual(261, objs[1].total_size)
-        self.assertEqual(197, objs[2].total_size)
-        self.assertEqual(68, objs[3].total_size)
-        self.assertEqual(12, objs[4].total_size)
-        self.assertEqual(12, objs[5].total_size)
-        self.assertEqual(29, objs[6].total_size)
-        self.assertEqual(44, objs[7].total_size)
-        self.assertEqual(257, objs[8].total_size)
 
     def test_compute_total_size_missing_ref(self):
         lines = list(_example_dump)
         # 999 isn't in the dump, not sure how we get these in real life, but
-        # they exist
-        lines.append('{"address": 8, "type": "tuple", "size": 16, "len": 1'
+        # they exist. we should live with references that can't be resolved.
+        lines[-1] = ('{"address": 8, "type": "tuple", "size": 16, "len": 1'
                      ', "refs": [999]}')
         manager = loader.load(lines, show_prog=False)
-        manager.compute_total_size()
+        obj = manager[8]
+        manager.compute_total_size(obj)
+        self.assertEqual(16, obj.total_size)
 
     def test_remove_expensive_references(self):
         lines = list(_example_dump)
@@ -396,3 +391,19 @@ class TestObjManager(tests.TestCase):
         manager = loader.load(_intern_dict_dump, show_prog=False)
         obj = manager.guess_intern_dict()
         self.assertEqual(8, obj.address)
+
+    def test_summarize_refs(self):
+        manager = loader.load(_example_dump, show_prog=False)
+        summary = manager.summarize(manager[2])
+        # Note that the dict itself is not excluded from the summary
+        self.assertEqual(['dict', 'int', 'str', 'tuple'],
+                         sorted(summary.type_summaries.keys()))
+        self.assertEqual(197, summary.total_size)
+
+    def test_summarize_excluding(self):
+        manager = loader.load(_example_dump, show_prog=False)
+        summary = manager.summarize(manager[2], excluding=[4, 5])
+        # No ints when they are explicitly filtered
+        self.assertEqual(['dict', 'str', 'tuple'],
+                         sorted(summary.type_summaries.keys()))
+        self.assertEqual(173, summary.total_size)
